/*
 * $QNXLicenseC:
 * Copyright 2007, 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#include	<stdio.h>
#include	<stdlib.h>
#include	<stddef.h>
#include	<string.h>
#include	<errno.h>
#include	<inttypes.h>
#include	<setjmp.h>
#include	<sys/types.h>
#include	<sys/mman.h>
#include	<x86/v86.h>
#include	<hw/pci.h>
#include	<hw/inout.h>
#include	<bios.h>
#include	<sys/rsrcdbmgr.h>
#include	<sys/rsrcdbmsg.h>
#include	<sys/syspage.h>
#include	<hw/sysinfo.h>
#include	<sys/slogcodes.h>

#define		X86_CFG_ADDR	0x0cf8
#define		X86_CFG_DATA	0x0cfc

#define		MAX_SLOTS		50

pdrvr_entry_t	bios_entry = {11, bios_attach, bios_detach, bios_cnfg_bridge,
								 bios_read_cnfg, bios_write_cnfg, bios_special_cycle,
								 bios_map_irq, bios_avail_irq, bios_map_addr,
								 bios_get_routing_options, bios_version};

uint32_t	irqlist [16] = {3, 4, 5, 6, 7, 9, 10, 11, 12, 14, 15, 0};

struct _pci_irq_routing {
	uint8_t						bus;
	uint8_t						device;
	struct _pci_irq_info {
		uint8_t					link;
		uint16_t				mask;
		} __attribute__ ((packed))
		 irq [4];
	uint8_t						slot;
	uint8_t						reserved;
};

typedef	struct {
	uint8_t			bus;
	uint8_t			devfn;
	uint16_t		irq_mask;
	} SLOT_MASK;

SLOT_MASK	*slot_mask;
int			num_slots = 0;

struct	{
	IRQRoutingOptionsBuffer	buf;
	uint8_t					databuf [2048];
	} route_buf;

static	uint64_t	free_mem_start, free_mem_end;
static	uint32_t	lastbus, version, hardware;

sigjmp_buf	env;

/**************************************************************************/
/* This is the actual call into the PCI bios                              */
/**************************************************************************/

static int _pci (uint32_t func, struct _v86reg *reg, void *data, int datasize)

{
	reg->eax = (PCI_FUNCTION_ID << 8) | func;
	if (_intr_v86 (0x1a, reg, data, datasize) != -1) {
		return ((reg->eax >> 8) & 0xff);
		}
	return (PCI_UNSUPPORTED_FUNCT);
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/

int		bios_read_cnfg (void *handle, uint32_t BusNumber, uint32_t DevFuncNumber,
						uint32_t Offset, uint32_t width, void *ConfigBuffer)

{
uint32_t		Addr, *Data = ConfigBuffer;
uint8_t			*pex_addr;
bios_dev_t		*pdev = handle;	

	if (sigsetjmp (env, SIGBUS) == 14)
		return (-1);

	if (pdev->pciex_addr == NULL) {
		Addr = (1 << 31) | (BusNumber << 16) | (DevFuncNumber << 8) | Offset;
		out32 (pdev->io_base, Addr);
		*Data = in32 (pdev->io_cnfg);
		out32 (pdev->io_base, 0);
		}
	else {
		pex_addr = pdev->pciex_addr + ((BusNumber << 20) | (DevFuncNumber << 12) | Offset);
		*Data = *(uint32_t *)pex_addr;
		}

	return (PCI_SUCCESS);
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/

int		bios_write_cnfg (void *handle, uint32_t BusNumber, uint32_t DevFuncNumber,
						uint32_t Offset, uint32_t width, const uint32_t Dword)

{
uint32_t		Addr;
uint8_t			*pex_addr;
bios_dev_t		*pdev = handle;	
	
	if (pdev->pciex_addr == NULL) {	
		Addr = (1 << 31) | (BusNumber << 16) | (DevFuncNumber << 8) | Offset;
		out32 (pdev->io_base, Addr);
		out32 (pdev->io_cnfg, Dword);
		out32 (pdev->io_base, 0);
		}
	else {
		pex_addr = pdev->pciex_addr + ((BusNumber << 20) | (DevFuncNumber << 12) | Offset);
		*(uint32_t *)pex_addr = Dword;
		}

	return (PCI_SUCCESS);
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/

int		bios_special_cycle (void *handle, uint32_t BusNumber, uint32_t SpecialCycleData)

{
struct	_v86reg reg;
int		stat;

	reg.ebx = BusNumber << 8;
	reg.edx = SpecialCycleData;
	reg.es	= 0;
	stat = _pci (PCI_GENERATE_SPECIAL_CYCLE, &reg, NULL, 0);
	return (stat);
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/

int		bios_map_irq (void *handle, uint32_t BusNumber, uint32_t DevFuncNumber,
					  uint32_t IRQNum, uint32_t IntPin, uint32_t Flags)

{
struct	_v86reg reg;

	memset (&reg, 0, sizeof (reg));
	reg.ebx = (BusNumber << 8) | (DevFuncNumber & ~0x3);
	reg.ecx = (Flags & PCI_MAP_IRQ) ? ((IRQNum << 8) | IntPin) : 0;
	reg.es	= 0;
	reg.ds = 0xf000;
	return (_pci (PCI_SET_IRQ, &reg, NULL, 0));
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/

int		bios_avail_irq (void *handle, uint32_t BusNumber, uint32_t DevFuncNumber,
						uint32_t *list, uint32_t *num_elements)

{
int			i, found;
uint32_t	dword = 0;
SLOT_MASK	*s = slot_mask;

	if ((i = bios_read_cnfg (handle, BusNumber, DevFuncNumber, offsetof (struct _pci_config_regs, Interrupt_Line),
							 4, &dword)) != PCI_SUCCESS) {
		return (i);
		}
	dword &= 0xff;
	if (dword > 0 && dword < 16) {
		*list = dword;
		*num_elements = 1;
		}
	else {
		for (i = found = 0; i < num_slots; i++, s++) {
			if (s->bus == BusNumber && s->devfn == DevFuncNumber) {
				found = 1;
				break;
				}
			}
		if (found) {
			int		cnt;
			for (i = cnt = 0; i < *num_elements && i < 16; i++) {
				if (s->irq_mask & (1 << i)) {
					*list++ = i;
					cnt++;
					}
				}
			i = cnt;
			}
		else {
			for (i = 0; i < *num_elements && irqlist [i]; i++) {
				*list++ = irqlist [i];
				}
			i--;
			}
		*num_elements = min (i, *num_elements);
		*list = 0;
		}

	return (PCI_SUCCESS);
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/

int 	bios_get_routing_options (struct _pci_route_buffer *IRQRoutingInfoBuffer,
									uint16_t *IRQForPCI)

{
struct	_v86reg reg;
int		stat, bsize;
struct	_pci_route_buffer *rptr;
uint8_t	*databuffer;

	memset (&reg, 0, sizeof (reg));
	bsize = sizeof (*IRQRoutingInfoBuffer) + IRQRoutingInfoBuffer->BufferSize;
	if ((rptr = malloc (bsize)) == NULL)
		return (-1);
	memset (rptr, 0, bsize);
	rptr->BufferSize = IRQRoutingInfoBuffer->BufferSize;
	rptr->DataBufferOffset = offsetof (struct _v86_memory, userdata) + sizeof (*IRQRoutingInfoBuffer);
	rptr->DataBufferSeg = 0;
	reg.ebx = 0;
	reg.es	= 0;
	reg.ds = 0xf000;
	reg.edi	= offsetof (struct _v86_memory, userdata);
	if ((stat = _pci (PCI_GET_IRQ_ROUTING_OPTIONS, &reg, rptr, bsize)) != PCI_SUCCESS) {
		free (rptr);
		return (stat);
		}
	databuffer = (uint8_t *) IRQRoutingInfoBuffer + sizeof (*IRQRoutingInfoBuffer);
	memcpy (databuffer, (uint8_t *) rptr + sizeof (struct _pci_route_buffer), min (IRQRoutingInfoBuffer->BufferSize, rptr->BufferSize));
	IRQRoutingInfoBuffer->BufferSize = rptr->BufferSize;
	free (rptr);
	if (IRQForPCI)
		*IRQForPCI = reg.ebx & 0xffff;
	return (stat);
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/

int		bios_cnfg_bridge (void *handle, uint32_t bus, uint32_t devfunc, pci_bus_t *pbus)

{
bios_dev_t		*pdev = handle;	

	if (pbus->pciex_addr != NULL) {
		pdev->pciex_addr = (uint8_t *)pbus->pciex_addr;
		pdev->pciex_size = pbus->pciex_size;
		}

	return (PCI_SUCCESS);
}

						  
/**************************************************************************/
/*                                                                        */
/**************************************************************************/

static	int		pci_bios_present (uint32_t *lastbus, uint32_t *version, uint32_t *hardware)

{
struct	_v86reg reg;
int		stat;

	memset (&reg, 0, sizeof (reg));
	stat = _pci (PCI_BIOS_PRESENT, &reg, NULL, 0);
	if(stat != PCI_SUCCESS || reg.edx != *((_uint32 *)"PCI "))
		return -1;
	*lastbus = reg.ecx & 0xff;
	*version = reg.ebx & 0xffff;
	*hardware = reg.eax & 0xff;
	return (stat);
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/

int		bios_map_addr (void *handle, uint64_t pciaddr, uint64_t *cpuaddr, uint32_t type)

{
	*cpuaddr = pciaddr;
	return (PCI_SUCCESS);
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/

int		bios_detach (void *handle)

{
bios_dev_t		*pdev;

	pdev = (bios_dev_t *) handle;
	free (pdev);
	return (EOK);
}

/**************************************************************************/
/*  Find routing for all IRQs for all slots and build a table of          */
/*  available IRQs.                                                       */
/**************************************************************************/

int		bios_get_irq_routing (uint16_t *irqval)

{
int			i, j, irqtemp [16];
int			status;
struct		_pci_irq_routing	*r;
SLOT_MASK	*s;
uint8_t		*p;

	memset ((char *) &route_buf, 0, sizeof (route_buf));
	route_buf.buf.BufferSize = sizeof (route_buf.databuf);
	if ((status = bios_get_routing_options ((struct _pci_route_buffer *) &route_buf, irqval)) != PCI_SUCCESS) {
		slogf (_SLOGC_PCI, _SLOG_INFO, "Get routing failed %x", status);
		return (status);
		}
	memset ((char *) &irqtemp, 0, sizeof (irqtemp));
	r = (struct _pci_irq_routing *) route_buf.databuf;

	for (i = 0, p = route_buf.databuf; i < 64; i++) {
		if (memcmp (p++, "\0", 1))
			break;
		}
	if (i >= 64) {
		slogf (_SLOGC_PCI, _SLOG_INFO, "routing buffer is NULL");
		return (status);
		}

	num_slots = (route_buf.buf.BufferSize / sizeof (*r)) * 4;
	if ((slot_mask = calloc (num_slots, sizeof (SLOT_MASK))) == NULL) {
		return (ENOMEM);
		}

	s = slot_mask;
	for (i = 0; i < (num_slots / 4); i++, r++) {
		s->bus = r->bus;
		for (j = 0; j < 4; j++) {
			s->devfn = (r->device | j);
			if (r->irq [j].link) {
				if (r->irq [j].mask) {
					int		b;

					s->irq_mask = r->irq [j].mask;
					for (b = 0; b < 16; b++) {
						if (r->irq [j].mask & (1 << b)) {
							irqtemp [b] = b;
							}
						}
					}
				}
			s++;
			}
		}
	for (i = j = 0; i < 16; i++) {
		if (irqtemp [i]) {
			irqlist [j++] = irqtemp [i];
			}
		}
	irqlist [j] = 0;

	return (status);
}

/*************************************************************************/
/*                                                                       */
/*************************************************************************/

int		bios_version (void *handle, uint32_t *lb, uint32_t *vers, uint32_t *hw)

{
	*lb = lastbus;
	*vers = version;
	*hw = hardware;
	return (PCI_SUCCESS);
}

/*************************************************************************/
/*                                                                       */
/*************************************************************************/

int		seed_resource (uint64_t *start, uint64_t *end, int type)

{
rsrc_alloc_t	ralloc;

	if (*start == 0 && *end == 0)
		return (EOK);
	memset ((char *) &ralloc, 0, sizeof (ralloc));
	ralloc.start = *start;
	ralloc.end = *end;
	ralloc.flags = type | RSRCDBMGR_FLAG_NOREMOVE;
//	if (verbose)
//		slogf (_SLOGC_PCI, _SLOG_INFO, "Seed Start %llx - End %llx - Flags %x", ralloc.start, ralloc.end, ralloc.flags);
	
	if (rsrcdbmgr_create (&ralloc, 1) == -1) {
		slogf (_SLOGC_PCI, _SLOG_ERROR, "Unable to seed resource");
		return (ENOMEM);
		}
	return (EOK);
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/

int		find_ram (void)

{
struct		asinfo_entry  *asinfo;
int			i, num, cnt;
uint64_t	highest_ram = 0;
struct	{
	uint64_t	start, end;
	} arr [50];

	num = _syspage_ptr->asinfo.entry_size / sizeof(*asinfo);
	asinfo = SYSPAGE_ENTRY (asinfo);

	for (i = cnt = 0; i < num; ++i) {
		char *name;
		if (asinfo->attr & AS_ATTR_KIDS || !asinfo->attr) {
			asinfo++;
			continue;
			}
		name = __hwi_find_string (asinfo->name);
		if (strcmp (name, "sysram") && strcmp (name, "rom")) {
			asinfo++;
			continue;
			}
		if (asinfo->start >= 0x100000000ULL) {	/* We don't want memory above 4GB */
			asinfo++;
			continue;
			}
		arr [cnt].start = asinfo->start;
		arr [cnt++].end = asinfo->end;
		if (! strcmp (name, "sysram")) {
			if (asinfo->end > highest_ram) {
				highest_ram = asinfo->end;
				}
			}
		asinfo++;
		}
	cnt--;
	while (1) {
		int			swap;
		uint64_t	tstart, tend;
		for (i = swap = 0; i < cnt; i++) {
			if (arr [i + 1].start < arr [i].start) {
				tstart = arr [i].start;
				tend = arr [i].end;
				arr [i].start = arr [i + 1].start;
				arr [i].end = arr [i + 1].end;
				arr [i + 1].start = tstart;
				arr [i + 1].end = tend;
				swap = 1;
				}
			}
		if (! swap)
			break;
		}
	for (i = 0; i <= cnt; i++) {
		if (arr [i].end < highest_ram) {
			continue;
			}
		if (i < cnt && (arr [i].end + 1 < arr [i + 1].start)) {
			free_mem_start = arr [i].end + 1;
			if (highest_ram = (free_mem_start % 0x100000)) {	//Some bioses report incorrect
				free_mem_start += (0x100000 - highest_ram);		//memory, so make mod. 1Meg
				}
			free_mem_end = arr [i + 1].start;
			}
		}
	if ((free_mem_start & 0xff00000) == 0xff00000)
		free_mem_start += 0x100000;
	return (0);
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/

void	busfault (int sig_number)

{
	siglongjmp (env, 14);
}

/**************************************************************************/
/*                                                                        */
/**************************************************************************/

int		bios_attach (char *options, void **handle)

{
uint32_t			irqtemp [16];
int					opt;
int					rval, i, status;
char				*value, *tok;
uint16_t			specd = 0, irqval = 0;
uint8_t				port61;
static bios_dev_t	*pdev;	
static char			*pci_drvr_opts[] = {
						"base", "cbase", "irqlist",
						NULL
						};

	signal (SIGBUS, busfault);
	memset ((char *) irqtemp, 0, sizeof (irqtemp));
	port61 = in8 (0x61);
	port61 |= 0x04;		/* Disable NMI */
	out8 (0x61, port61);
	if ((status = pci_bios_present (&lastbus, &version, &hardware)) != PCI_SUCCESS) {
		slogf (_SLOGC_PCI, _SLOG_ERROR, "pci-bios: No BIOS present - Status %x - errno %d!", status, errno);
		if (hwi_find_item (HWI_NULL_OFF, "hw", "pci", NULL) != HWI_NULL_OFF) {
			slogf (_SLOGC_PCI, _SLOG_INFO, "pci-bios: HWI reports PCI, so continuing");
			}
		else {
			slogf (_SLOGC_PCI, _SLOG_ERROR, "pci-bios: HWI failed!");
			return (ENODEV);
			}
		}
	else {
		if (hardware & 0x02) {		//Check for configuration mechanism 2
			slogf (_SLOGC_PCI, _SLOG_ERROR, "Hardware mechanism 2 not supported");
			return (ENOSYS);
			}
		}

	if ((pdev = calloc (1, sizeof (bios_dev_t))) == NULL) {
		return (ENOMEM);
		}

	while (options && *options != '\0') {
		if ((opt = getsubopt (&options, pci_drvr_opts, &value)) == -1 || value == NULL) {
			continue;
			}
		switch (opt) {
			case	0:
				pdev->io_base = strtoul (value, 0, 0);
				break;

			case	1:
				pdev->io_cnfg = strtoul (value, 0, 0);
				break;

			case	2:
				specd = 1;
				for (i = 0, tok = value - 1; i < (sizeof (irqlist) / sizeof (uint32_t)); i++) {
					rval = strtol (tok + 1, &tok, 10);
					irqtemp [i] = rval;
					if (rval == 0)
						break;
					}
			}
		}

	if (! pdev->io_base) {
		pdev->io_base = X86_CFG_ADDR;
		}

	if (! pdev->io_cnfg) {
		pdev->io_cnfg = X86_CFG_DATA;
		}

	find_ram ();
	if (seed_resource (&free_mem_start, &free_mem_end, RSRCDBMGR_PCI_MEMORY))
		return (1);

	rval = ENOMEM;
	*handle = NULL;

	status = bios_get_irq_routing (&irqval);

	if (specd) {
		i = 0;
		while (irqtemp [i]) {
			irqlist [i] = irqtemp [i];
			i++;
			}
		irqlist [i] = 0;
		}

	*handle = pdev;
	rval = EOK;

	return (rval);
}
