/*
 * $QNXLicenseC:
 * Copyright 2007, 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


#ifndef	_BIOS_H_INCLUDED
#define	_BIOS_H_INCLUDED

#include	<sys/platform.h>
#include	<pcidrvr.h>

// PCI BIOS function call numbers

#define PCI_FUNCTION_ID                 0xb1
#define PCI_BIOS_PRESENT                0x01
#define PCI_FIND_DEVICE                 0x02
#define PCI_FIND_CLASS                  0x03
#define PCI_GENERATE_SPECIAL_CYCLE      0x06
#define PCI_READ_CONFIG_BYTE            0x08
#define PCI_READ_CONFIG_WORD            0x09
#define PCI_READ_CONFIG_DWORD           0x0a
#define PCI_WRITE_CONFIG_BYTE           0x0b
#define PCI_WRITE_CONFIG_WORD           0x0c
#define PCI_WRITE_CONFIG_DWORD          0x0d
#define PCI_GET_IRQ_ROUTING_OPTIONS     0x0e
#define PCI_SET_IRQ                     0x0f

#define BIOS_MEM_BASE_ADDR		0xd0000000		// default addresses
#define BIOS_MEM_SPACE_LEN		0x1fffffff
#define	BIOS_IO_BASE_ADDR		0x1000
#define BIOS_IO_SPACE_LEN		0xefb0
#define BIOS_ISA_SPACE_LEN		0x00800000

typedef struct _bios_dev {
	uint32_t		io_base;
	uint32_t		io_cnfg;
	void			*base;
	void			*cnfg;
	uint64_t		pci_cnfg;
	uint64_t		pci_isa;
	uint64_t		pci_io;
	uint64_t		pci_mem;
	uint8_t			revision;
	uint8_t			rsvd[3];
	uint8_t			*pciex_addr;
	uint32_t		pciex_size;
	} bios_dev_t;

// Function Prototypes

int		bios_read_cnfg (void *, _uint32, _uint32, _uint32, _uint32, void *);
int		bios_write_cnfg (void *, _uint32, _uint32, _uint32, _uint32, const _uint32);
int		bios_special_cycle (void *, _uint32, _uint32);
int		bios_map_irq (void *, _uint32, _uint32, uint32_t, uint32_t, uint32_t);
int		bios_avail_irq (void *, _uint32, _uint32, uint32_t *, uint32_t *);
int		bios_cnfg_bridge (void *, _uint32, _uint32, pci_bus_t *);
int		bios_map_addr (void *, _uint64, _uint64 *, _uint32);
int		bios_detach (void *);
int		bios_attach (char *, void **);
int 	bios_get_routing_options (IRQRoutingOptionsBuffer *, uint16_t *);
int		bios_version (void *handle, uint32_t *lb, uint32_t *vers, uint32_t *hw);

#endif
