/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


#include "startup.h"

const struct callout_slot callouts[] = {
	{ CALLOUT_SLOT( reboot, _pckbd ) },
};

const struct debug_device debug_devices[] = {
	{ 	"console",
		{	"",
			NULL
		},
		init_vga,
		put_vga,
		{	&display_char_vga,
			&poll_key_pckbd,
		}
	},
	{ 	"8250",
		{	"0x3f8^0.57600.1843200.16",
			"0x2f8^0.57600.1843200.16"
		},
		init_8250,
		put_8250,
		{	&display_char_8250,
			&poll_key_8250,
			&break_detect_8250,
		}
	},
};

static void
set_sc400_irq() {
	unsigned	pirq;
	unsigned	rirq;
	unsigned	v;

	pirq = strtoul(optarg, &optarg, 10);
	if((optarg[0] == ',') && (pirq <= 7)) {
		rirq = strtoul(optarg+1, NULL, 10);
		out8(0x22, 0xd4 + (pirq >> 1));
		v = in8(0x23);
		if(pirq & 1) {
			v = (v & 0x0f) | rirq << 4;
		} else {
			v = (v & 0xf0) | rirq;
		}
		out8(0x23, v);
	}
}

//
// This startup program must execute out of RAM. It performs 2 functions.
//
// 1. It gathers information about the system and places it in a structure
//    called the system page. The kernel references this structure to
//    determine everything it needs to know about the system. This structure
//    is also available to user programs (read only if protection is on)
//    via _syspage_ptr->.
//
// 2. It (optionally) turns on the MMU and starts the next program
//    in the image file system. 
//
int
main(int argc, char **argv, char **envv) {
	int 		opt;
	int 		reserve = 1;
	size_t		shadow_rom = 0;
	unsigned	top_prio_irq = 3;
	int			lapic_intrs = 0;
	extern unsigned	mem_scan;
		
	//
	// Make sure the A20 line is in a good state.
	//
	x86_init_pcbios();

	add_callout_array(callouts, sizeof(callouts));

	while((opt = getopt(argc, argv, COMMON_OPTIONS_STRING "bI:i:Lms:")) != -1) {
		switch(opt) {
		case 'b':
			reserve = 0;
			break;
		case 'I':
			top_prio_irq = getsize(optarg,0);
			break;
		case 'i':
			set_sc400_irq();
			break;
		case 'L':
			lapic_intrs = 1;
			break;
		case 'm':
			mem_scan = MEG(64);
			break;
		case 's':
			shadow_rom = getsize(optarg, 0);
			if(shadow_rom == 0) shadow_rom = 1;
			break;
		default:
			handle_common_option(opt);
			break;
		}
	}

	//
	// Initialize debugging output (kprintf won't work before this point).
	//
	select_debug(debug_devices, sizeof(debug_devices));

	//
	// Collect information on all free RAM in the system.
	//
	init_raminfo();

	//
	// We normally stay away from the 1st 4K of memory. It is needed
	// to implement virtual 86 mode. If this capability is not needed
	// then this memory does not need to be allocated making it available.
	//
	if(reserve) alloc_ram(0L, 4096, 1);

	//
	// Find out how many AP's we've got (must be done before init_mmu).
	//
	init_smp();

	//
	// In a virtual system we need to initialize the page tables
	//
	if(shdr->flags1 & STARTUP_HDR_FLAGS1_VIRTUAL) init_mmu();

	if(shadow_rom != 0) x86_pcbios_shadow_rom(0xc0000, shadow_rom);

	// ---------------------------------------------------------------------//
	//                                                                      //
	// The following routines have hardware or system dependancies which    //
	// may need to be changed on a non-PC compatible system. The default    //
	// routines assume a BIOS is present. If you do not have a bios you     //
	// will need to replace these routines with ones of your own.           //
	//                                                                      //
	init_intrinfo_8259(top_prio_irq);
	if(lapic_intrs) {
		init_intrinfo_apic();
	}
	init_qtime();
	init_cacheattr();
	init_cpuinfo();
	init_hwinfo();

	add_typed_string(_CS_MACHINE, "x86pc");
	//LATER: add_typed_string(_CS_HW_PROVIDER, Do_a_ROM_scan());
	//LATER: add_typed_string(_CS_HW_SERIAL, Do_a_ROM_scan());
	//                                                                      //
	// ---------------------------------------------------------------------//

	//
	// Load the bootstrap executables in the image filesystem and
	// initialize various syspage pointers. This must be the *last*
	// initialization done before transfering to the next program.
	//
	init_system_private();

	//
	// This is real handy for debugging a new version of this startup
	// program. Commenting this line out will save a whack of code.
	//
	print_syspage();
	return 0;
}
