/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */




/*
 * X86 interrupt classes
 *

 *
 */
#ifndef __X86_INTR_H_INCLUDED
#define __X86_INTR_H_INCLUDED

/*
 * X86 specific interrupt classes
 */
#define	X86_INTR_CLASS_NMI			(0x8000UL << 16)
#define X86_INTR_NMI				(X86_INTR_CLASS_NMI + 0)

#define	X86_INTR_CLASS_APIC			(0x8001UL << 16)
#define X86_INTR_APIC_TIMER			(X86_INTR_CLASS_APIC + 0)
#define X86_INTR_APIC_THERMAL		(X86_INTR_CLASS_APIC + 1)
#define X86_INTR_APIC_PERFORMANCE	(X86_INTR_CLASS_APIC + 2)
#define X86_INTR_APIC_LINT0			(X86_INTR_CLASS_APIC + 3)
#define X86_INTR_APIC_LINT1			(X86_INTR_CLASS_APIC + 4)
#define X86_INTR_APIC_ERROR			(X86_INTR_CLASS_APIC + 5)

/*
** Definitions for all i386 GDT and IDT entries. Missing entries are invalid.
**
** GDT - 32 entries
**
** 00 -               08 - GDT itself     10 - IDT            18 - 
** 20 - sysseg0       28 - sysseg1        30 - sysseg2        38 - sysseg3
** 40 - sysseg4       48 - sysseg5        50 - sysseg6        58 - sysseg7
** 60 -               68 -                70 -                78 - Phys page 1 (V86)
** 80 - Kernel CS(0)  88 - Kernel DS(0)   90 - Proc CS(1)     98 - Proc DS(1)
** A0 - LDT           A8 -                B0 -                B8 -
** C0 - User CS(3)    C8 - User DS(3)     D0 -                D8 -
** E0 -               E8 -                F0 -                F8 -
*/
/*
** LDT - 5 entries
** There is only one LDT for the system and it is overlaid ontop of the GDT
** starting at GDT entry A8.
**
** 04 -               0C -                14 -                1C - User CS(3)
** 24 - User DS(3)
*/
/*
** IDT - 48 entries plus whatever is used for hardware interrupts
** vec
** 00  000 - Div 0        008 - Debug       010 - NMI          018 - Brkpoint
** 04  020 - INTO         028 - BOUND       030 - ILL OPCODE   038 - Dev NA
** 08  040 - Double fault 048 - reserved    050 - Inv TSS      058 - Seg NP
** 0c  060 - Stack fault  068 - GP fault    070 - Page fault   078 - reserved
** 10  080 - FPU error    088 - Alignment   090 - Reserved     098 - Reserved
** 14  0A0 - Reserved     0A8 - Reserved    0B0 - Reserved     0B8 - Reserved
** 18  0C0 - Reserved     0C8 - Reserved    0D0 - Reserved     0D8 - Reserved
** 1c  0E0 - Reserved     0E8 - Reserved    0F0 - Reserved     0F8 - Reserved
** 20  100 - Kdebug trap  108 - Kdebug msg  110 -              118 -         
** 24  120 -              128 -             130 - SMP IPI      138 - SMP IPI
** 28  140 - Kernel call  148 -             150 -              158 -
** 2c  160 -              168 -             170 -              178 - APIC spurious
** 30  180 - Hardware vectors....
*/

#endif

/* __SRCVERSION("intr.h $Rev: 1 $"); */
