/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */




.text

/*
;
; We're starting an application processor in an SMP system. The processor
; is in real mode. Our mission, if we chose to accept it, is to get it
; into 32-bit protect mode and transfer control to an address (apstart32) 
; in the startup code.
;
*/
.code16

	.globl	apstart16_start
	.globl	apstart16_end
	.globl	apstart32_ptr

apstart16_start:
		// Set stack to top of intr table. We're going to cream the top word.
		cli
		xorw	%ax,%ax
		movw	%ax,%ss
		movw	$0x400,%sp	
		
		movw	%cs,%ax
		movw	%ax,%ds
		xorl	%ebx,%ebx
		.byte 	0xe8,0x00,0x00		// call	l1
l1:
		
		popw	%bx				// Find where we are.
		addl	$(null_entry - l1), %ebx	// point bx at null_entry
		
//
//	Switch processor into protected mode.  
//
//	The following selectors are defined:
//
//		8	- data selector for 0-4G
//		16	- code selector for 0-4G
//

//
//	Calculate the linear address of the GDT area.
//
		xorl	%eax,%eax
		movw	%ds,%ax
		shll	$4,%eax
		addl	%ebx,%eax
					
//
//	Set up the GDT descriptor.
//
		.byte	0xc7,0x07,0x18,0x00	// movw	$24,0(%bx)			// limit
		.byte	0x66,0x89,0x47,0x02 // movl	%eax,2(%bx)			// base
		.byte	0x66,0x0f,0x01,0x17 // lgdt	(%bx)
					
//
//	Now switch the machine into protected mode by setting
//	bit 0 of CR0.  Enable the cache for write-back mode.
//
		movl	%cr0,%eax
		orb		$1,%al
		movl	%eax,%cr0
		jmp		flush
flush:
		
		.byte	0x66,0xea	// 16:32 far jump opcode
apstart32_ptr:
		.long	0			// go to 32apstart.S for remainder of initialization
		.short	16
		
	   	.align	2	

null_entry:
		.byte	0,0,0,0,0,0,0,0
data_entry:
		.short	0xffff				// limit
		.short	0					// base_low
		.byte	0					// base_high_low
		.byte	0x93				// flags
		.byte	0xcf				// flags + limit
		.byte	0					// base_high_high
code_entry:
		.short	0xffff				// limit
		.short	0					// base_low
		.byte	0					// base_high_low
		.byte	0x9b				// flags
		.byte	0xcf				// flags + limit
		.byte	0					// base_high_high
apstart16_end:
