/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"
#include <x86/intr.h>
#include "lapic.h"

//
// Initialize interrupt controller hardware & intrinfo structure in the
// system page.
// This code is hardware dependant and may have to be changed
// changed by end users.
//
// This version is for the Pentium local APIC


extern struct callout_rtn	interrupt_id_apic;
extern struct callout_rtn	interrupt_eoi_apic;
extern struct callout_rtn	interrupt_mask_apic;
extern struct callout_rtn	interrupt_unmask_apic;
extern struct callout_rtn	interrupt_config_apic;

extern struct callout_rtn	interrupt_id_apic_spurious;
extern struct callout_rtn	interrupt_eoi_apic_spurious;
extern struct callout_rtn	interrupt_mask_apic_spurious;
extern struct callout_rtn	interrupt_unmask_apic_spurious;
extern struct callout_rtn	interrupt_config_apic_spurious;


static struct startup_intrinfo apic_intrs[] = {
	{ X86_INTR_CLASS_APIC, 6, _NTO_INTR_SPARE, 6, 1, 
		INTR_FLAG_SMP_BROADCAST_MASK|INTR_FLAG_SMP_BROADCAST_UNMASK,
		{INTR_GENFLAG_NOGLITCH, 0, &interrupt_id_apic},
		{0, 0, &interrupt_eoi_apic},
		&interrupt_mask_apic, 
		&interrupt_unmask_apic, 
		&interrupt_config_apic,
	},
	{ X86_INTR_CLASS_APIC+0xffff, 1, _NTO_INTR_SPARE, APIC_SPURIOUS_INTERRUPT, 1, 
		0,
		{0, 0, &interrupt_id_apic_spurious},
		{0, 0, &interrupt_eoi_apic_spurious},
		&interrupt_mask_apic_spurious, 
		&interrupt_unmask_apic_spurious, 
		&interrupt_config_apic_spurious,
	},
};

void
lapic_configure() {
	unsigned	hw_intr;

	#define LVT_LINT (LVT_TRIGGER_MODE_LEVEL|LVT_POLARITY_LOW|LVT_DELIVERY_MODE_FIXED)

	if(lapic_vaddr != NULL) {
		hw_intr = apic_intrs[0].cpu_intr_base;
		apic_write(LAPIC_LTR,  (LVT_MASKED|LVT_TIMER_MODE_PERIODIC)|(hw_intr+0));
		apic_write(LAPIC_LTMR, (LVT_MASKED|LVT_DELIVERY_MODE_FIXED)|(hw_intr+1));
		apic_write(LAPIC_LPCR, (LVT_MASKED|LVT_DELIVERY_MODE_FIXED)|(hw_intr+2));
//		
// Don't configure LINTx vectors - one is configured to serve as
// the INTR pin and the other is NMI. This will change once we start
// start supporting IOAPIC's.
//		apic_write(LAPIC_LL0R, (LVT_MASKED|LVT_LINT)|(hw_intr+3));
//		apic_write(LAPIC_LL1R, (LVT_MASKED|LVT_LINT)|(hw_intr+4));
//		
		apic_write(LAPIC_LER,  (LVT_MASKED)|(hw_intr+5));

		apic_write(LAPIC_SVR,  SVR_APIC_ENABLED | APIC_SPURIOUS_INTERRUPT);
	}
}

void
init_intrinfo_apic() {
	unsigned	hw_intr;

	if(lapic_vaddr != NULL) {
		hw_intr = find_top_cpu_intr();
		if(hw_intr == 0) {
			hw_intr = FIRST_HARDWARE_INTERRUPT-1;
		}
		hw_intr += 1;
		apic_intrs[0].cpu_intr_base = hw_intr;

		apic_intrs[0].patch_data = lapic_vaddr;
		apic_intrs[1].patch_data = lapic_vaddr;

		lapic_configure();

		add_interrupt_array(apic_intrs, sizeof(apic_intrs));
	}
}

__SRCVERSION("apic_intr.c $Rev: 1 $");
