/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */




/*
;
; This is code to cycle an AP into the kernel startup code for an SMP system.
; This code MUST be position independant.
;
*/

#include "callout.ah"
		
/*
;
; EDI has a pointer to the kernel syspage pointer
; ECX has the cpu number
;
; As the kernel finishes initializing each processor, it will store the
; starting address of the AP initializing code into 'ap_start_addr' 
; in the smpinfo structure. Since XCHG is always an atomic operation, one
; and only one of the AP's whirling around in the loop here will pick
; up the address and jump to it.
;
; Things are a tad complex because we're supporting both the old
; X86 specific callout section and the new CPU independent one. Somewhere
; down the road, we can remove the gear for the X86 specific section.
;
*/
CALLOUT_START(smp_spin, 0, 0)
    xorl    %eax,%eax
    xorl    %edx,%edx
	xorl	%esi,%esi
	cmpw	$0,SYSPAGE_X86_SMPINFO+2(%edi)
	je		1f
    movw    SYSPAGE_X86_SMPINFO(%edi),%dx
	leal	(%edx,%edi),%esi
1:
    movw    SYSPAGE_SMP(%edi),%dx
    addl    %edx,%edi
	movl	%eax,SMP_PENDING(%edi) // Tell startup next AP can go
1:
    xchgl   SMP_START_ADDR(%edi),%eax
    orl     %eax,%eax
	jne		2f
	orl		%esi,%esi
	je		1b
    xchgl   X86_AP_START_ADDR(%esi),%eax
    orl     %eax,%eax
    je      1b
2:
    cmpl    $-1,%eax
    je      shutdown
    // The CPU number is in ECX, but since we want the kernel to be
    // able to tell if it's got a newer startup or not, we switch it
    // with the starting address we got told to use. This way, the 
    // kernel can test EAX on entry and if it's < 256, it knows that
    // it's got a startup that passes the cpu number to it.
    xchgl	%ecx,%eax
    jmp     *%ecx
	
shutdown:		// kernel doesn't want us :-(, sniff, sniff.
    movl      %eax,SMP_START_ADDR(%edi)	// shutdown next AP
    hlt       
    jmp       shutdown
CALLOUT_END(smp_spin)
