/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */




/*
;
; This is a kernel hardware callout.
; It may be called from within an interrupt handler.
; This code MUST be position independant.
;
*/

#include "callout.ah"

/*
 * Interrupts are always off in this code.
 *
 * Note that interrupt_id_* and interrupt_eoi_* are not actually
 * called by the kernel. For performance reasons, they are instead
 * copied into place and intermixed with other kernel code. As such,
 * they don't follow normal calling conventions. They should just fall out 
 * their bottoms when done rather than attempting to do a return 
 * instruction. The 'id' routine should return the interrupt level in
 * the EAX register. 
 * By turning on various INTR_GENFLAG_* bits in the intrinfo_entry,
 * the following values can be loaded before entry to this code.
 * 
 *		syspage_ptr			=> EBX
 *		intrinfo_entry		=> ECX
 *		intr mask count		=> EDX (only available in EOI routine)
 *		cpu number			=> ESI
 *
 * If the INTR_FLAG_CPU_FAULT bit is on, the eoi routine should set
 * the EAX register to the following value to indicate the exception
 * that has occurred:
 * 		EAX = (fault_number << 16) | (signal_code << 8) | (signal_number)
 * Set EAX to zero if no exception has occured. If the exception is memory
 * related, set EDX to referenced address that caused the exception, zero
 * if there is no address.
 */

/*
; 
; int interrupt_id()
;
; Returns: 0 - it's not a glitch   -1 - it's a glitch
;
*/
CALLOUT_START(interrupt_id_8259m, 0, 0)
	//Nothing to do
CALLOUT_END(interrupt_id_8259m)

CALLOUT_START(interrupt_id_8259m7, 0, 0)
    xorl      %eax,%eax
    inb       $0x20,%al		// we default to ISR visible
	shrb	  $0x07,%al
    xorb      $0x01,%al		// flip glitch bit
	negl	  %eax
CALLOUT_END(interrupt_id_8259m7)

CALLOUT_START(interrupt_id_8259s, 0, 0)
    movb      $0x20,%al		//EOI master
    outb      %al,$0x20
CALLOUT_END(interrupt_id_8259s)

CALLOUT_START(interrupt_id_8259s7, 0, 0)
    movb      $0x20,%al		//EOI master
    outb      %al,$0x20
    xorl      %eax,%eax
    inb       $0xa0,%al		// we default to ISR visible
	shrb	  $0x07,%al
    xorb      $0x01,%al		// flip glitch bit
	negl	  %eax
CALLOUT_END(interrupt_id_8259s7)


/*
; 
; void interrupt_eoi
;
; Send an end-of-interrupt to the PIC.
;
*/
CALLOUT_START(interrupt_eoi_8259m, 0, 0)
    movb      $0x20,%al
    outb      %al,$0x20
CALLOUT_END(interrupt_eoi_8259m)

CALLOUT_START(interrupt_eoi_8259s, 0, 0)
    movb      $0x20,%al
    outb      %al,$0xa0
CALLOUT_END(interrupt_eoi_8259s)

/*
;
; int interrupt_unmask(volatile struct syspage_entry *syspage, int intr)
;
; Unmask the passed hardware interrupt number. 
;
*/
CALLOUT_START(interrupt_unmask_8259m, 0, 0)
	movl 	  8(%esp),%ecx
    movb      $0x01,%ah
    shlb      %cl,%ah
    inb       $0x21,%al
    notb      %ah
    andb      %ah,%al
    outb      %al,$0x21
    xorl      %eax,%eax
    ret       
CALLOUT_END(interrupt_unmask_8259m)

CALLOUT_START(interrupt_unmask_8259m7, 0, 0)
    inb       $0x21,%al
    andb      $0x7f,%al
    outb      %al,$0x21
    xorl      %eax,%eax
    ret       
CALLOUT_END(interrupt_unmask_8259m7)

CALLOUT_START(interrupt_unmask_8259s, 0, 0)
	movl 	  8(%esp),%ecx
    movb      $0x01,%ah
    shlb      %cl,%ah
    inb       $0xa1,%al
    notb      %ah
    andb      %ah,%al
    outb      %al,$0xa1
    xorl      %eax,%eax
    ret       
CALLOUT_END(interrupt_unmask_8259s)

CALLOUT_START(interrupt_unmask_8259s7, 0, 0)
    inb       $0xa1,%al
    andb      $0x7f,%al
    outb      %al,$0xa1
    xorl      %eax,%eax
    ret       
CALLOUT_END(interrupt_unmask_8259s7)
	

/*
;
; int interrupt_mask(volatile struct syspage_entry *syspage, int intr)
;
; Mask the passed hardware interrupt number.
;
*/
CALLOUT_START(interrupt_mask_8259m, 0, 0)
	movl 	  8(%esp),%ecx
    movb      $0x01,%ah
    shlb      %cl,%ah
    inb       $0x21,%al
    orb		  %ah,%al
    outb      %al,$0x21
    xorl      %eax,%eax
    ret       
CALLOUT_END(interrupt_mask_8259m)

CALLOUT_START(interrupt_mask_8259m7, 0, 0)
    inb       $0x21,%al
    orb       $0x80,%al
    outb      %al,$0x21
    xorl      %eax,%eax
    ret       
CALLOUT_END(interrupt_mask_8259m7)

CALLOUT_START(interrupt_mask_8259s, 0, 0)
	movl 	  8(%esp),%ecx
    movb      $0x01,%ah
    shlb      %cl,%ah
    inb       $0xa1,%al
    orb       %ah,%al
    outb      %al,$0xa1
    xorl      %eax,%eax
    ret       
CALLOUT_END(interrupt_mask_8259s)

CALLOUT_START(interrupt_mask_8259s7, 0, 0)
    inb       $0xa1,%al
    orb       $0x80,%al
    outb      %al,$0xa1
    xorl      %eax,%eax
    ret       
CALLOUT_END(interrupt_mask_8259s7)

/*
;
; int interrupt_config(volatile struct syspage_entry *syspage,
                        struct intrinfo_entry *info, int intr)
;
; Return configuration information for interrupt
;
; Interrupt level 2 is a cascaded interrupt, so we disallow attaching
; to it.
;
*/
CALLOUT_START(interrupt_config_8259m, 0, 0)
    xor     %eax,%eax
    cmpl    $2,0xc(%esp)
    jne     1f
    mov     $INTR_CONFIG_FLAG_DISALLOWED,%eax
1:
    ret
CALLOUT_END(interrupt_config_8259m)
