/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */

#include "startup.h"
#include <x86/inout.h>
#include <sys/syspage.h>

#define PCI_CONFADDR		0xcf8
#define PCI_CONFDATA		0xcfc

#define PCI_ADDR(bus, dev, func, reg) \
	( 0x80000000 | (((bus)  & 0xff) << 16) | \
                       (((dev)  & 0x1f) << 11) | \
                       (((func) & 0x07) <<  8) | \
                       (((reg)  & 0xff)      ))

#define GET_ECX_APIC_CORESIZE(reg)	(((reg) & (0x3 << 12)) >> 12)
#define GET_ECX_NC(reg)				((reg) & 0xef)
#define AMD_PMM7_CPULowPwrEn 1 << 0
#define AMD_PMM7_NBLowPwrEn 1 << 1


static inline int8_t pci_getb(paddr32_t addr)
{
	out32(PCI_CONFADDR, addr & ~3);
	return in8(PCI_CONFDATA | (addr & 3));
}

static inline void pci_setb(paddr32_t addr, uint8_t val)
{
	out32(PCI_CONFADDR, addr & ~3);
	out8(PCI_CONFDATA | (addr & 3), val);
}

static inline void cpuid(unsigned *a, unsigned *b, unsigned *c, unsigned *d, unsigned in)
{
	asm ("cpuid"
			: "=a" (*a),
			"=b" (*b),
			"=c" (*c),
			"=d" (*d)
			: "a" (in));
}

static int amdk8_num_processors(void)
{
	unsigned eax, ebx, ecx, edx;
	unsigned num_cpus, core_id_size, cores_per_cpu;

	num_cpus = lsp.syspage.p->num_cpu;
	if ( num_cpus == 1 ) {
		return 0;
	}

	cpuid( &eax, &ebx, &ecx, &edx, 0x80000008 );
	core_id_size = GET_ECX_APIC_CORESIZE(ecx);
	if ( core_id_size == 0 ) {
		if ( !(cpu_features & X86_FEATURE_HTT) ) {
			return 1;
		} else {
			cores_per_cpu = GET_ECX_NC(ecx) + 1;
		}
	} else {
		cores_per_cpu = (1 << core_id_size);
	}
	return num_cpus/cores_per_cpu;
}

void fix_amdk8_tsc(void)
{
	uint8_t val;
	int		proc, num_procs;

	num_procs = amdk8_num_processors();
	if ( debug_flag ) {
		kprintf("fixing TSC drift for %d AMD K8 processors\n", num_procs );
	}
	for ( proc = 0; proc < num_procs; proc++ ) {

		val = pci_getb(PCI_ADDR(0, 0x18 + proc, 3, 0x87));

		if ( debug_flag ) {
			kprintf("Northbridge %d: PMM7 is currently %x\n", proc, val );
		}

		val &= ~(AMD_PMM7_CPULowPwrEn|AMD_PMM7_NBLowPwrEn);

		pci_setb(PCI_ADDR(0, 0x18 + proc, 3, 0x87), val);
	}
}

void check_tsc(void)
{
	char		id[13];
	unsigned	family, eax, ebx, ecx, edx;

	cpuid( &eax, &ebx, &ecx, &edx, 0 );
	memcpy( &id[0], &ebx, 4 );
	memcpy( &id[4], &edx, 4 );
	memcpy( &id[8], &ecx, 4 );
	id[12] = 0;

	cpuid( &eax, &ebx, &ecx, &edx, 1 );
	family = (eax >> 8) & 0xf;

	if ( !strcmp( id, "AuthenticAMD" ) && family == 0xf ) {
		fix_amdk8_tsc();
	}
	/* all others are considered safe for now */
}

__SRCVERSION("check_tsc.c $Rev: 1 $");
