/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"

void
print_x86_boxinfo() {
	struct x86_boxinfo_entry *box = lsp.cpu.x86_boxinfo.p;

	kprintf("  hw_flags:%l\n", box->hw_flags);
}

void
print_x86_diskinfo() {
	struct x86_diskinfo_entry *disk = lsp.cpu.x86_diskinfo.p;
	int num = lsp.cpu.x86_diskinfo.size / sizeof(*disk);
	int i;
	
	for(i = 0; i < num; i++) {
		if(disk[i].valid) {
			kprintf("  %d)  cyls:%x heads:%x sectors:%x nblocks:%l\n",
				i,
				disk[i].cyls,
				disk[i].heads,
				disk[i].sectors,
				disk[i].nblocks
			);
		}
	}
}


void
print_x86_gdt() {
	struct x86_seg_descriptor_entry	*gdt = TOPTR(gdt_paddr);
	struct x86_seg_descriptor_entry	*p;
	int i, num;

	kprintf("  base:%l\n", lsp.syspage.p->un.x86.gdt);
	for(i = num = 0, p = gdt ; i < 32; ++i, ++p) {
		unsigned long	limit;

		if(p->flags) {
			if(num >= 2) {
				num = 0;
				kprintf("\n");
			}

			limit = p->limit + ((unsigned long)(p->limflags & 0xf) << 16);
			if(p->limflags & X86_G_BIT) {
				limit *= 0x1000;
				limit += 0x0fff;
			}
			kprintf("  off:%b base:%l limit:%l", i * 8,
				(((unsigned long)p->base_xhi) << 24) +
				(((unsigned long)p->base_hi) << 16) +
				(unsigned long)p->base_lo, limit);
			++num;
		}
	}

	kprintf("\n");
}

void
print_x86_idt() {
	kprintf("  base:%l\n", lsp.syspage.p->un.x86.idt);
}

void
print_x86_smpinfo() {
	unsigned	i;
	struct x86_smpinfo_entry *smp = lsp.cpu.x86_smpinfo.p;

	kprintf("  lapic_addr:%l lapicid map:", smp->lapic_addr);
	for(i = 0; i < NUM_ELTS(smp->lapicid_to_index); ++i) {
		if(smp->lapicid_to_index[i] != (unsigned char)~0) {
			kprintf(" %d>%d", i, smp->lapicid_to_index[i]);
		}
	}
	kprintf("\n");
}


static uint64_t
read_entry(paddr32_t p, unsigned i) {
	if(paddr_bits > 32) {
		return(((uint64_t *)p)[i]);
	}
	return(((uint32_t *)p)[i]);
}

void
print_x86_pgdir() {
	paddr32_t 	pagedir = page_tables;
	int			i, j, num;
	uint64_t	pde, pte;
	
	if(!(shdr->flags1 & STARTUP_HDR_FLAGS1_VIRTUAL)) return;

	kprintf("  Page Directory %l", pagedir);
	if(paddr_bits > 32) kprintf( ": PAE");
	kprintf("\n");

	if(debug_flag > 3) {
		unsigned	pd_entries = 1024;
		unsigned	pt_entries = 1024;

		if(paddr_bits > 32) {
			pd_entries = 2048;
			pt_entries = 512;
		}
		for(i = num = 0 ; i < pd_entries; ++i) {
			pde = read_entry(pagedir, i);
			if(pde) {
				if(num >= 4) {
					num = 0;
					kprintf("\n");
				}
	
				kprintf("%x-%w%l  ", i, (unsigned)(pde >> 32) & 0xffff, (unsigned)pde);
	
				if(debug_flag > 4) {
					if(pde & X86_PDE_PS) {
						kprintf("BIG!\n");
						num = -1;
					} else {
						kprintf("\n  ");
						for(j = num = 0 ; j < pt_entries; ++j) {
							pte = read_entry(pde & ~0x3ff, j);
							if(pte) {
								if(num >= 4) {
									num = 0;
									kprintf("\n  ");
								}
		
								kprintf("%x-%w%l  ", j, (unsigned)(pte >> 32) & 0xffff, (unsigned)pte);
		
								++num;
							}
		
						}
						kprintf("\n");
						num = -1;
					}
				}
				++num;
			}
		}
		kprintf("\n");
	}
}

__SRCVERSION("cpu_debug.c $Rev: 1 $");
