/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include <x86/inline.h>
#include <x86/priv.h>
#include <x86/cpu.h>

/********************************************************************
*********************************************************************

	These definitions are required for the system independent code.

*********************************************************************
********************************************************************/

#define KERCALL_SEQUENCE(name)	uint8_t name[] = {		\
					0xcd, 0x28,		/* int 0x28 */		\
					0xcd, 0x20,		/* int 0x20 */		\
}

#define CPU_SYSPAGE_TYPE	SYSPAGE_X86

struct cpu_local_syspage {
	SYSPAGE_SECTION(x86_boxinfo);
	SYSPAGE_SECTION(x86_smpinfo);
	SYSPAGE_SECTION(x86_diskinfo);
	SYSPAGE_TYPED_SECTION(paddr32_t, pgdir);
};

extern void print_x86_gdt(void);
extern void print_x86_idt(void);
extern void print_x86_boxinfo(void);
extern void print_x86_diskinfo(void);
extern void print_x86_smpinfo(void);
extern void print_x86_pgdir(void);

#define CPU_PRT_SYSPAGE_RTNS				\
	CPU_PRT_SYSPAGE_RTN(x86, 1, boxinfo),	\
	CPU_PRT_SYSPAGE_RTN(x86, 1, diskinfo),	\
	CPU_PRT_SYSPAGE_RTN(x86, 1, smpinfo),	\
	CPU_PRT_SYSPAGE_RTN(x86, 0, gdt),		\
	CPU_PRT_SYSPAGE_RTN(x86, 0, idt),		\
	CPU_PRT_SYSPAGE_RTN(x86, 0, pgdir),

#define BOOTSTRAPS_RUN_ONE_TO_ONE	0

// Have startup avoid allocating low memory for itself - bios stuff there
#define CPU_MEM_AVOID_START		0
#define CPU_MEM_AVOID_SIZE		KILO(16)

#define CPU_COMMON_OPTIONS_STRING	"Bx"

/********************************************************************
*********************************************************************

	Everything below is specific to the X86.

*********************************************************************
********************************************************************/

#define GDT_SIZE 	(sizeof(struct x86_seg_descriptor_entry) * 40)
#define FIRST_HARDWARE_INTERRUPT	0x30
// Use a hardcoded value to ensure we handle the P6 and Pentium restriction
// that the bottom 4 bits of the vector have to be all one's.
#define APIC_SPURIOUS_INTERRUPT		0x2f

#define PARASIZE		16
#define ROUNDPA(_x)		ROUND ((_x), PARASIZE)


#define REAL_VADDR		0xFEC00000		// Virtual address of 0-1M physical
#define FREE_VADDR		(REAL_VADDR + MEG(1) + KILO(64)) // First free virtual addr

//
// Defines for hw_ns486_cs
//
#define NS486_CS_IO			0x00
#define NS486_CS_MEM		0x80
#define NS486_CS_16BIT		0x40
#define NS486_CS_CMD_DELAY	0x08
#define NS486_CS_WS(w)		(w)

//
// Function prototypes
//

// "init_gdt.c"
void init_gdt(unsigned idt_size);

// "x86_mmu_4m.c"
void x86_init_4m_region(int global);

// "processor.c"
uintptr_t x86_map(uintptr_t vaddr,paddr_t paddr,paddr_t size,int flags);
uintptr_t x86_4m_map(uintptr_t vaddr,paddr_t paddr,paddr_t size,int flags);
void x86_init_pgdir(void);
void x86_load_pgdir(unsigned cpu);

// "hw_cputype.s"
unsigned x86_cputype(void);

// "hw_cpuspeed.c"
unsigned long x86_cpuspeed(unsigned cputype);

// "hw_fputype.s"
unsigned x86_fputype(void);

// "hw_cpuid_string.c"
unsigned long x86_cpuid(int, void *);
int x86_cpuid_string(char *buf,int max);

// "hw_enable_a20.c"
int x86_enable_a20(unsigned long cpu,int only_keyboard);

// "hw_ns486_cs.c"
int x86_ns486_cs(int cs, unsigned long addr, unsigned long mask, unsigned int flags, unsigned int external);

// "hw_scanmem.c"
void x86_scanmem(paddr32_t beg, paddr32_t end);

// "set_desc32.c"
void set_desc32(struct x86_seg_descriptor_entry *, int, int, uintptr_t, uintptr_t);

void x86_init_pcbios(void);
int  x86_pcbios_shadow_rom(paddr32_t rom, size_t size);
void init_one_cpuinfo(unsigned cpunum);
void set_cycles_per_sec(struct qtime_entry *);
void x86_v86_add_mapping(paddr32_t start, paddr32_t len);
void init_intrinfo_8259(unsigned top_prio_irq);
int	load_elf32mmu_4m(paddr32_t, Elf32_Ehdr *, Elf32_Phdr *);

//
// Variable prototypes
//
extern paddr32_t			gdt_paddr;
extern paddr32_t			page_tables;
extern unsigned long 		cpu_features;
extern uint8_t				cpu_features_set;
extern X86_CPU_REGISTERS	*boot_regs;
extern uint8_t				acpi_ignore;
extern paddr32_t			lapic_paddr;
extern void					*lapic_vaddr;

//
// board specific SMP stuff
//
// Intel MultiProcessor Specification versions
unsigned imps_smp_num_cpu(void);

// ACPI Specification versions
unsigned acpi_smp_num_cpu(void); // returns zero if ACPI not present


// APIC versions
void apic_set_paddr(paddr_t p);
void apic_add_cpuid(unsigned num_cpus, unsigned id);
void apic_smp_init(struct smp_entry *, unsigned num_cpus);
int apic_smp_start(unsigned cpu, void (*start)(void));
unsigned apic_smp_adjust_num(unsigned);
void lapic_configure(void);
void init_intrinfo_apic(void);
void check_tsc(void);


unsigned	find_top_cpu_intr(void);

//
// Common debug routines
//
void init_vga(unsigned channel, const char *init, const char *defaults);
void put_vga(int c);
extern struct callout_rtn	display_char_vga;
extern struct callout_rtn	poll_key_pckbd;

//
// Common callouts
//
extern struct callout_rtn	reboot_pckbd;
extern struct callout_rtn	reboot_fastbios;
extern struct callout_rtn	timer_load_8254;
extern struct callout_rtn	timer_reload_8254;
extern struct callout_rtn	timer_value_8254;

extern struct callout_rtn	interrupt_id_8259m;
extern struct callout_rtn	interrupt_eoi_8259m;
extern struct callout_rtn	interrupt_mask_8259m;
extern struct callout_rtn	interrupt_unmask_8259m;
extern struct callout_rtn	interrupt_config_8259m;
extern struct callout_rtn	interrupt_id_8259m7;
extern struct callout_rtn	interrupt_mask_8259m7;
extern struct callout_rtn	interrupt_unmask_8259m7;

extern struct callout_rtn	interrupt_id_8259s;
extern struct callout_rtn	interrupt_eoi_8259s;
extern struct callout_rtn	interrupt_mask_8259s;
extern struct callout_rtn	interrupt_unmask_8259s;
extern struct callout_rtn	interrupt_id_8259s7;
extern struct callout_rtn	interrupt_mask_8259s7;
extern struct callout_rtn	interrupt_unmask_8259s7;

extern struct callout_rtn	interrupt_id_nmi;
extern struct callout_rtn	interrupt_eoi_nmi;
extern struct callout_rtn	interrupt_mask_nmi;
extern struct callout_rtn	interrupt_unmask_nmi;

extern struct callout_rtn	interrupt_id_nmi_bios;
extern struct callout_rtn	interrupt_eoi_nmi_bios;
extern struct callout_rtn	interrupt_mask_nmi_bios;
extern struct callout_rtn	interrupt_unmask_nmi_bios;
extern struct callout_rtn	interrupt_config_nmi_bios;

/* __SRCVERSION("cpu_startup.h $Rev: 1 $"); */
