/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"
#include <sys/elf_nto.h>

extern Elf32_Phdr *is_elf(Elf32_Ehdr *hdr, paddr32_t addr);

#define PAGEBITS 	(__PAGESIZE-1)
#define PADDR_BASE	(__PAGESIZE*1024)
#define PADDR_SIZE	(__PAGESIZE*1024)
#define PADDR_MASK	(PADDR_SIZE-1)

int
load_elf32mmu_4m(paddr32_t addr, Elf32_Ehdr *hdr, Elf32_Phdr *phdr) {
	static paddr32_t	paddr_base;
	static unsigned		vaddr_base;
	int					i;

	// Do we support 4M pages, or is it use-in-place?
	if(((lsp.cpuinfo.p[0].flags & X86_CPU_PSE) == 0) ||
		(shdr->imagefs_paddr == shdr->image_paddr + shdr->startup_size)) {
		return 0;
	}

	for(i = 0; i < hdr->e_phnum; ++i, ++phdr)  {
		switch(phdr->p_type) {
		case PT_LOAD:
			{
				uintptr_t vaddr  = phdr->p_vaddr & ~PAGEBITS;
				paddr32_t daddr = phdr->p_paddr & ~PAGEBITS;

				if((vaddr & PADDR_MASK) != (daddr & PADDR_MASK) || (phdr->p_memsz != phdr->p_filesz)) {
					if(vaddr_base) { 
						// Unmap so we can remap using 4k pages
						x86_4m_map(vaddr_base, 0, PADDR_SIZE, 0);
					}
					return 0;
				}
				if(vaddr_base == 0) {
					unsigned bits;

					// First time around, set up 4m mapping
					vaddr_base = vaddr & ~PADDR_MASK;
					paddr_base = daddr & ~PADDR_MASK;
					bits = X86_PTE_PRESENT|X86_PTE_WRITE|X86_PDE_PS;
					if(lsp.cpuinfo.p[0].flags & X86_CPU_PGE) {
						bits |= X86_PTE_GLOBAL;
					}
					x86_4m_map(vaddr_base, paddr_base, PADDR_SIZE, bits);
				} else if((vaddr & ~PADDR_MASK) != vaddr_base) {
					crash("executables don't fit in one 4m region");
				}
			}
			break;
		}
	}

	return 1;
}

__SRCVERSION("elf_x86.c $Rev: 1 $");
