/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"


static const unsigned aamtime[][2] = {
	{ 686, 16 },	/* Not really needed as RDTSC is suported */
	{ 586, 23 },	/* Not really needed as RDTSC is suported */
	{ 486, 22 },
	{ 386, 32 }
};

unsigned long
x86_cpuspeed(unsigned cpunum) {
	unsigned x;	/* change in counter */
	unsigned long loops, incr;
	unsigned long i;
	unsigned long bias = aamtime[0][1];
	unsigned long cpufreq;
	unsigned start;
	unsigned ticks;

	if( lsp.cpuinfo.p[cpunum].flags & X86_CPU_RDTSC ) {
	    uint64_t rdtsc_ticks1, rdtsc_ticks2;

		ticks = timer_ns2tick(1000000); /* ticks in 1ms */
		start = timer_start();
		rdtsc(&rdtsc_ticks1);
		__asm__ __volatile__ (".align 16");
		while(timer_diff(start) < ticks)
			;
		rdtsc(&rdtsc_ticks2);
		return( ((unsigned long)rdtsc_ticks2 - (unsigned long)rdtsc_ticks1) / 1000 );
	}

	for (i=0; i < sizeof aamtime/sizeof(aamtime[0]); i++) {
		if (aamtime[i][0] == lsp.cpuinfo.p[cpunum].cpu) {
			bias = aamtime[i][1];
			break;
		}
	}

	/* find biggest loop that doesn't wrap */
	incr = 16000000L/(bias*100*2);
	x = 2000;

	ticks = timer_ns2tick(2000000); /* get #ticks in 2ms */

	/*
	 * We increase the iterations until it takes 1ms to perform the loop.
	 *
	 * It is based on ascii adjust after multiple, fairly stable since
	 * nobody uses it:-).
	 */
	__asm__ __volatile__ (".align 16");
	for(loops = incr; loops < 0x10000L; loops += incr) {
		start = timer_start();
		__asm__ __volatile__(
			".align 16; 1: aam; loopl 1b"
			:
			: "c" (loops)
			: "eax"
		);
		x = timer_diff(start);
		if(x > ticks) break;
	}

	/*
 	 * X is the number of ticks.
         * Loops*bias is the number of CPU clocks.
	 */
	cpufreq = (loops*bias*1000)/(timer_tick2ns(x));
	return cpufreq;
}

__SRCVERSION("hw_cpuspeed.c $Rev: 1 $");
