/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"

#define WAIT_8042() while(in8(0x64) & 0x02)

//
// We test that the A20 line works by making sure that address 0 and 1M
// do not map to the same physcial memory location.
//
static int
test_gate_a20(unsigned long cpu) {
	unsigned short	save1, save2;
	unsigned short	*addr1 = 0,
					*addr2 = (unsigned short *) (1024*1024);

	save1 = *addr1;
	save2 = *addr2;
	*addr1 = 0;
	*addr2 = ~0;

	if(cpu >= 486)
		flushcache();

	if(*addr1 == *addr2)
		return(-1);

	*addr1 = save1;
	*addr2 = save2;
	return(0);
	}


//
// Enable the A20 line. Many (most) PC compatible systems come up with
// address line A20 forced to zero. This addressed a bug in early DOS.
// We can never escape history :-(
//
// This code is hardware dependant and may have to be
// changed by end users.
//
int
x86_enable_a20(unsigned long cpu, int only_keyboard) {
	unsigned char byte;
	volatile unsigned n;

	//
	// If A20 is already enabled just return ok.
	//
	if(test_gate_a20(cpu) == 0)
		return(0);

	//
	// Use the fast gate A20 mechanism supported by many of the new
	// chip-sets.
	//
	if(only_keyboard == 0) {
		byte = in8(0x92);
		io_safe(); io_safe();
		out8(0x92, byte | 0x02);
		io_safe(); io_safe();
		if(test_gate_a20(cpu) == 0)
			return(0);
	}

	//
	// Use the keyboard micro-controller to enable A20.
	//
	WAIT_8042();
	out8(0x64, 0xd1); io_safe(); io_safe();
	WAIT_8042();
	out8(0x60, 0xdf); io_safe(); io_safe();
	WAIT_8042();
	// Give some time to take effect
	for(n = 0xffff ; n ; --n)
		;

	//
	// If the A20 line was not enabled the system will behave in confusing
	// and difficult to understand ways. Therefore we verify that it
	// did indeed work!
	//
	return(test_gate_a20(cpu));
}

__SRCVERSION("hw_enable_a20.c $Rev: 1 $");
