/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"

static void modb (int port, unsigned int andval, unsigned int orval) {
	out8(port, in8(port) & andval | orval);
}

int
x86_ns486_cs (int cs, unsigned long addr, unsigned long mask, unsigned int flags, unsigned int external) {
	//
	// chip selects start at 1
	//
	--cs;
	
	//
	// disable the chip select while modifying it
	//
	modb (0xef02, ~(1 << cs), 0);
	
	//
	// set base address and mask
	//
	out16 (0xef04 + (cs * 4), addr);
	out16 (0xef06 + (cs * 4), addr >> 16);
	out16 (0xef24 + (cs * 4), mask);
	out16 (0xef26 + (cs * 4), mask >> 16);
	
	//
	// set access time and command delay flag
	//
	if (cs & 1) {
		modb (0xef50 + (cs >> 1), 0x0f, (flags & 0xf) << 4);
	} else {
		modb (0xef50 + (cs >> 1), 0xf0, flags & 0xf);
	}
	
	//
	// set I/O or memory type
	//
	if (flags & NS486_CS_MEM) {
		modb (0xef03, 0xff, 1 << cs);
	} else {
		modb (0xef03, ~(1 << cs), 0);
	}
	
	//
	// set forced 16-bit
	//
	if (flags & NS486_CS_16BIT) {
		modb (0xef57, 0xff, 1 << cs);
	} else {
		modb (0xef57, ~(1 << cs), 0);
	}
	
	//
	// set external chip select
	//
	modb (0xef45 + external - 1, 0xff, 1 << cs);
		
	//
	// enable the chip select
	//
	modb (0xef02, 0xff, 1 << cs);
	
	return 0;
}

__SRCVERSION("hw_ns486_cs.c $Rev: 1 $");
