/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"

// Temporary hack until the new <x86/syspage.h> has propagated everywhere
// Can be removed after a suitable interval of time 2008/08/20
#if !defined(X86_CPU_SSE2)
#define X86_CPU_SSE2 (1 << 15)
#endif

//
// Initialize cpuinfo structures in the system page.
// This code is hardware independent and should not have to be changed
// changed by end users.
//

static int
enable_nx(void) {
	unsigned	flags;
	uint32_t	info[3];

	flags = x86_cpuid(0x80000000, 0);
	if(flags >= 0x80000001) {
		x86_cpuid(0x80000001, info);
		if(info[1] & (1 << 20)) {
			// Chip supports NX, turn it on
			wrmsr(X86_MSR_EFER, rdmsr(X86_MSR_EFER) | X86_EFER_NXE);
			return 1;
		}
	}
	return 0;
}

void
init_one_cpuinfo(unsigned cpunum) {
	struct cpuinfo_entry 	*cpu;
	unsigned long			flags = 0;
	unsigned				name;
	unsigned				cpu_type;
	char					buf[128];
	
	//
	// Run an instruction maze to determine the cpu type.
	//
	cpu_type = x86_cputype();

	//
	// Set cpu->name
	//
	if(x86_cpuid_string(buf, sizeof buf) == -1) {
		ultoa_end(cpu_type,buf,10);
	}
	name = add_string(buf);
	cpu = &lsp.cpuinfo.p[cpunum];
	cpu->name = name;
	cpu->cpu = cpu_type;
	if(cpu_features_set) {
		flags |= X86_CPU_CPUID;
		if(cpu_features & X86_FEATURE_TSC) flags |= X86_CPU_RDTSC;
		if(cpu_features & X86_FEATURE_MMX) flags |= X86_CPU_MMX;
		if(cpu_features & X86_FEATURE_CMOV) flags |= X86_CPU_CMOV;
		if(cpu_features & X86_FEATURE_FPU) flags |= CPU_FLAG_FPU;
		if(cpu_features & X86_FEATURE_PSE) flags |= X86_CPU_PSE;
		if(cpu_features & X86_FEATURE_PAE) flags |= X86_CPU_PAE;
		if(cpu_features & X86_FEATURE_MTRR) flags |= X86_CPU_MTRR;
		if(cpu_features & X86_FEATURE_PGE) flags |= X86_CPU_PGE;
		if(cpu_features & X86_FEATURE_SEP) flags |= X86_CPU_SEP;
		if(cpu_features & X86_FEATURE_FXSR) flags |= X86_CPU_FXSR;
		if(cpu_features & X86_FEATURE_SIMD) flags |= X86_CPU_SIMD;
		if(cpu_features & X86_FEATURE_SSE2) flags |= X86_CPU_SSE2;
		if(enable_nx()) flags |= X86_CPU_NX;
		check_tsc();
	} else {
		if(x86_fputype()) flags |= CPU_FLAG_FPU;
	}
	if(cpu->cpu >= 486) {
		flags |= X86_CPU_WP | X86_CPU_INVLPG | X86_CPU_BSWAP;
	}
	if(cpu->cpu >= 586) {
		unsigned cr4 = rdcr4();

		if(flags & X86_CPU_PGE)  cr4 |= X86_CR4_PGE;
		if(flags & X86_CPU_PSE)  cr4 |= X86_CR4_PSE;
		if((flags & X86_CPU_PAE) && (paddr_bits > 32))  cr4 |= X86_CR4_PAE;
		if(flags & X86_CPU_FXSR)  cr4 |= X86_CR4_OSFXSR;
		ldcr4(cr4);
	}
	if(flags & X86_CPU_PSE) x86_init_4m_region(flags & X86_CPU_PGE);

	if(shdr->flags1 & STARTUP_HDR_FLAGS1_VIRTUAL) flags |= CPU_FLAG_MMU;

	//
	// Set cpu->flags.
	//
	cpu->flags = flags;

	cpu->speed = (cpu_freq != 0) ? (cpu_freq / 1000000) : x86_cpuspeed(cpunum);
}

void
init_cpuinfo() {
	unsigned				num;
	unsigned				i;
	struct cpuinfo_entry	*cpu;

	num = lsp.syspage.p->num_cpu;
	cpu = set_syspage_section(&lsp.cpuinfo, sizeof(*lsp.cpuinfo.p) * num);
	for(i = 0; i < num; ++i) {
		cpu[i].ins_cache = system_icache_idx;
		cpu[i].data_cache = system_dcache_idx;
	}
	init_one_cpuinfo(0);
}

__SRCVERSION("init_cpuinfo.c $Rev: 1 $");
