/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"
#include "multiboot.h"

//
// Initialize hwinfo structure in the system page.
// This code is hardware dependant and may have to be changed
// changed by end users.
//

#define get_port(base, i)	(((uint16_t *)(base))[i])

static void
add_diskinfo(unsigned bus) {
	struct hwi_diskgeometry			*di;

	if(find_startup_info(NULL, STARTUP_INFO_DISK) != NULL) {
		const struct startup_info_disk	*disk;

		hwi_alloc_item(HWI_TAG_INFO(group), HWI_ITEM_DEVCLASS_DISK, bus);
		disk = NULL;
		for( ;; ) {
			disk = find_startup_info(disk, STARTUP_INFO_DISK);
			if(disk == NULL) break;
			di = hwi_alloc_tag(HWI_TAG_INFO(diskgeometry));
			di->disknumber = disk->drive;
			di->sectorsize = 9;
			di->heads = disk->heads;
			di->cyls = disk->cylinders;
			di->sectors = disk->sectors;
			if(disk->blocks != 0) {
				di->nblocks = disk->blocks;
			} else {
				di->nblocks = (unsigned long)di->heads * di->cyls * di->sectors;
			}
		}
	} else if(boot_regs->eax == MB_SIGNATURE) {
		// Check for Multiboot style information....
		struct mb_info	*info = (void *)boot_regs->ebx;

		if(info->flags & MB_INFO_FLAG_DRIVES_VALID) {
			unsigned				len = info->drives_length;

			if(len > 0) {
				struct mb_info_drive	*dr_info = (void *)info->drives_addr;

				hwi_alloc_item(HWI_TAG_INFO(group), HWI_ITEM_DEVCLASS_DISK, bus);
				do {
					di = hwi_alloc_tag(HWI_TAG_INFO(diskgeometry));
					di->disknumber = dr_info->drive_number;
					di->sectorsize = 9;
					di->heads = dr_info->drive_heads;
					di->cyls = dr_info->drive_cylinders;
					di->sectors = dr_info->drive_sectors;
					di->nblocks = (unsigned long)di->heads * di->cyls * di->sectors;
	
					len -= dr_info->size;
   					dr_info = (void *)((uintptr_t)dr_info + dr_info->size);
				} while(len != 0);
			}
		}
	}
}

void
init_hwinfo() {
	const struct startup_info_box	*sbox;
	char							*bus;
	unsigned						root;
	hwi_tag							*tag;
	unsigned						i;
	unsigned						bus_owner;
	
	root = hwi_find_item(HWI_NULL_OFF, HWI_ITEM_ROOT_HW, NULL);
	bus = HWI_ITEM_BUS_ISA;
	sbox = find_startup_info(NULL, STARTUP_INFO_BOX);
   	if(sbox != NULL) {
		switch(sbox->bustype) {
		case 1:
			bus = HWI_ITEM_BUS_MCA;
			break;
		case 2:
			tag = hwi_alloc_item(HWI_TAG_INFO(bus), HWI_ITEM_BUS_PCI, root);
			hwi_alloc_item(HWI_TAG_INFO(device), "bios", hwi_tag2off(tag));
			break;
		case 3:
			bus = HWI_ITEM_BUS_EISA;
			break;
		}
	}
	bus_owner = hwi_tag2off(hwi_alloc_item(HWI_TAG_INFO(bus), bus, root));
	add_diskinfo(bus_owner);

	// Add keyboard device
	hwi_add_device(bus, HWI_ITEM_DEVCLASS_MISC, "pckbd", 0);
	hwi_add_irq(1);
	hwi_add_location(0x60, 5, 0, hwi_find_as(0x60, 0));

	// Add serial device(s)
	for(i = 0; i < 4; ++i) {
		uint16_t	port;

		port = get_port(0x400, i);
		if(port == 0) break;
		if(i == 0) {
			hwi_add_device(bus, HWI_ITEM_DEVCLASS_SERIAL, "8250", 0);
		}
		hwi_add_irq(4 - (i & 1));
		hwi_add_location(port, 7, 0, hwi_find_as(port, 0));
	}

	// Add parallel device(s)
	for(i = 0; i < 3; ++i) {
		uint16_t	port;

		port = get_port(0x408, i);
		if(port == 0) break;
		if(i == 0) {
			hwi_add_device(bus, HWI_ITEM_DEVCLASS_PARALLEL, "centronics", 0);
		}
		hwi_add_location(port, 7, 0, hwi_find_as(port, 0));
	}

	// Keep the allocator away from where things may be installed
	hwi_add_device(HWI_ITEM_BUS_UNKNOWN, HWI_ITEM_DEVCLASS_MISC, "in-use", 0);
	hwi_add_location(0, 0x200, 0, hwi_find_as(0, 0));
	hwi_add_location(KILO(64)-0x100, 0x100, 0, hwi_find_as(0, 0)); //buggy
	hwi_add_location(KILO(640), 0x34000, 0, hwi_find_as(KILO(640), 1));
	hwi_add_location(0xd6000, 0x2a000, 0, hwi_find_as(0xd6000, 1));
	hwi_add_irq(14);
	hwi_add_irq(15);

//NYI: temp implementation
{ extern	void old_init_hwinfo(); old_init_hwinfo(); }
}

__SRCVERSION("init_hwinfo.c $Rev: 1 $");
