/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"


//
// Add a mapping into real mode memory for V86 mode
// This code is hardware independant and should not have to be
// changed by end users.
//
void
x86_v86_add_mapping(paddr32_t start, paddr32_t len) {
	paddr32_t	end = start + len;

	start = TRUNCPG(start);
	x86_map(REAL_VADDR+start, start, end-start, X86_PTE_PRESENT|X86_PTE_WRITE);
}

extern void apic_set_vaddr(void) __attribute__((weak));
void 
apic_set_vaddr(void) {
	// Dummy version - overridden by apic_paddr.c if needed
}

//
// Initialize page tables. This prepares the system to leap into virtual mode.
// This code is hardware independant and should not have to be
// changed by end users.
//
void
init_mmu() {
	extern		void exec386();
	paddr32_t	paddr;
	int			i;
	unsigned	ram;
	paddr32_t	ebda_start;
	paddr_t		curr;

	x86_init_pgdir();
		
	//
	// We map a bunch of special physical pages to high virtual addresses.
	//

	// Map the first page of physical memory.
	x86_v86_add_mapping(0, __PAGESIZE);

	// Find the start of the extended BIOS data area
	ebda_start = 0;
	ram = AS_NULL_OFF;
	for( ;; ) {
		ram = as_find(ram, "ram", NULL);
		if(ram == AS_NULL_OFF) break;
		curr = as_off2info(ram)->end + 1;
		if((curr < KILO(640)) && curr > ebda_start) ebda_start = curr;
	}
	if(ebda_start == 0) ebda_start = KILO(636);
	ebda_start = TRUNCPG(ebda_start);


	// Map the adaptor region on a PC.
	x86_v86_add_mapping(ebda_start, MEG(1) - ebda_start);

	// We also map first page above 1M back to physical address 0 to
	// simulate real mode wrapping of addresses. This is needed for V86 mode.
	x86_map(REAL_VADDR + MEG(1), 0, __PAGESIZE, X86_PTE_PRESENT | X86_PTE_WRITE);

	// Map all other pages below 1M+64K to a page of ROM.   This is for
	// the benefit of buggy BIOS's which would otherwise fault.
	for (i = __PAGESIZE; i < ebda_start; i += __PAGESIZE) {
		x86_map(REAL_VADDR + i, 0xff000, __PAGESIZE, X86_PTE_PRESENT | X86_PTE_WRITE);
	}
	for (i = MEG(1) + __PAGESIZE; i < MEG(1) + KILO(64); i += __PAGESIZE) {
		x86_map(REAL_VADDR + i, 0xff000, __PAGESIZE, X86_PTE_PRESENT | X86_PTE_WRITE);
	}
	
	// Map the code which turns on the pager. This switcher code also moves
	// the stack into this page. This ensures no discontinuity when the pager
	// is enabled.
	paddr = TRUNCPG((paddr32_t) &exec386);
	x86_map(paddr, paddr, __PAGESIZE*2, X86_PTE_PRESENT | X86_PTE_WRITE | X86_PTE_USER);

	// Tell the rest of the system where the real mode memory is.
	lsp.syspage.p->un.x86.real_addr = (void *)REAL_VADDR;

	apic_set_vaddr();
}

__SRCVERSION("init_mmu.c $Rev: 1 $");
