/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"

#define PC_CLOCK_RATE	838095345UL				/* period in femtoseconds */

//
// Initialize qtime structure in the system page.
// This code is hardware dependant and may have to be
// changed by end users.
//

static const struct callout_slot timer_callouts[] = {
	{ CALLOUT_SLOT( timer_load, _8254 ) },
	{ CALLOUT_SLOT( timer_value, _8254 ) },
/*
	This routine is optional. You need it if your timer chip can not
	be set into a free-running mode (e.g., you need to reload it with
	a new countdown value, or something magical has to be done to clear
	the interrupt condition). It is also used when another device shares
	the timer chip's interrupt level.

	{ CALLOUT_SLOT( timer_reload, _8254 ) },
*/
};

static inline unsigned
get_timer() {
	unsigned ret;

	__asm__ __volatile__(
		"xorl	%%eax,%%eax;"
		"outb	%%al,$0x43;"
		"inb	$0x40,%%al;"
		"movb	%%al,%%ah;"
		"inb	$0x40,%%al;"
		"xchgb	%%al,%%ah;"
		: "=a" (ret)
	);
	return ret;
}

static unsigned
timer8254_start() {
	return(get_timer());
}

static unsigned
timer8254_diff(unsigned start) {
	unsigned now = get_timer();

	//Subtraction appears backwards because the chip counts _down_.
	if(now > start) start += KILO(64);
	return(start - now);
}

void
init_qtime() {
	struct qtime_entry *qtime = alloc_qtime();

	if(timer_freq == 0) {
		qtime->timer_rate = PC_CLOCK_RATE;
		qtime->timer_scale = -15; /* timer_rate is in femtoseconds */
	} else {
		invert_timer_freq(qtime, timer_freq);
	}

	//
	// Initialize the timer chip for rate division and full period countdown.
	//
	out8(0x43, 0x34);
	out8(0x40, 0);
	out8(0x40, 0);

	timer_start = timer8254_start;
	timer_diff  = timer8254_diff;

	add_callout_array(timer_callouts, sizeof(timer_callouts));

	set_cycles_per_sec(qtime);
}

__SRCVERSION("init_qtime.c $Rev: 1 $");
