/*
 * $QNXLicenseC:
 * Copyright 2007, 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


#include "startup.h"


//
// Initialize diskinfo structure in the system page.
// This code is system dependant and may have to be
// changed by end users.
//
static void
do_diskinfo() {
	struct x86_diskinfo_entry		*diskinfo;
	const struct startup_info_disk	*disk;
	int								lastdrive;

	lastdrive = -1;
	disk = NULL;
	for( ;; ) {
		disk = find_startup_info(disk, STARTUP_INFO_DISK);
		if(disk == NULL) break;
		if(disk->drive > lastdrive) lastdrive = disk->drive;
	}
	if(lastdrive >= 0) {
		struct x86_diskinfo_entry		*di;

		diskinfo = set_syspage_section(&lsp.cpu.x86_diskinfo, (lastdrive+1)*sizeof(*lsp.cpu.x86_diskinfo.p));
		disk = NULL;
		for( ;; ) {
			disk = find_startup_info(disk, STARTUP_INFO_DISK);
			if(disk == NULL) break;
			di = &diskinfo[disk->drive];
			di->valid = 512;
			di->heads = disk->heads;
			di->cyls = disk->cylinders;
			di->sectors = disk->sectors;
			if(disk->blocks != 0) {
				di->nblocks = disk->blocks;
			} else {
				di->nblocks = (unsigned long)di->heads * di->cyls * di->sectors;
			}
		}
	}
}

static void
do_boxinfo() {
	struct x86_boxinfo_entry *box = set_syspage_section(&lsp.cpu.x86_boxinfo, sizeof(*lsp.cpu.x86_boxinfo.p));
	unsigned wd, bus;
	const struct startup_info_box	*sbox;
	
	wd = X86_HW_WATCHDOG_NONE;
	bus = X86_HW_BUS_ISA;
	//
	// Set box->hw_flags
	//
	sbox = find_startup_info(NULL, STARTUP_INFO_BOX);
   	if(sbox != NULL) {
		switch(sbox->bustype) {
		case 0:	bus = X86_HW_BUS_ISA;						break;
		case 1:	bus = X86_HW_BUS_MCA;						break;
		case 2: bus = X86_HW_BUS_PCI|X86_HW_BUS_ISA; 		break;
		case 3: bus = X86_HW_BUS_EISA;						break;
		}
		switch(sbox->boxtype) {
		case 80:
		case 60:
		case 50: wd = X86_HW_WATCHDOG_PS2; break;
		}
	}
	box->hw_flags = bus | wd;
	if(bus & X86_HW_BUS_ISA) {
		struct meminfo_entry	*m;

		for(m = lsp.meminfo.p; m->type != MEMTYPE_NONE; ++m) {
			if((m->type == MEMTYPE_RAM) && ((m->addr + m->size) > MEG(16))) {
				/* ISA cards can't reach some memory for DMA */
				box->hw_flags |= X86_HW_NO_DMA_MEM;
			}
		}
	}
}

//
// Initialize boxinfo structure in the system page.
// This code is hardware dependant and may have to be changed
// changed by end users.
//
void
old_init_hwinfo() {
	//NYI: temp implementation
	do_boxinfo();
	do_diskinfo();
}

__SRCVERSION("old_hwinfo.c $Rev: 1 $");
