/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"
	
typedef uint32_t	pte32_t;
typedef uint64_t	pte64_t;

paddr32_t page_tables;
	
uintptr_t
x86_map(uintptr_t vaddr, paddr_t paddr, paddr_t size, int flags) {
	static uintptr_t	free_addr = FREE_VADDR;
	unsigned			n;
	uintptr_t			mapped_vaddr;

	if(!(shdr->flags1 & STARTUP_HDR_FLAGS1_VIRTUAL)) {
		return(paddr);
	}
	size = ROUNDPG(size);
	if(vaddr == ~0L) {
		/* don't care where the memory is, use next free vaddr */
		vaddr = free_addr;
		free_addr += size;
	}
	mapped_vaddr = vaddr;
//kprintf("map(%x,%x,%x,%x)\n", vaddr, paddr, size, flags);
	paddr |= flags;
	for(n = size >> 12; n > 0; --n) {
		if(paddr_bits > 32) {
			pte64_t			*pte;
			pte64_t			*pde;

			pde = (pte64_t *)(page_tables + ((vaddr >> 21) << 3));
			if((*pde & X86_PTE_PRESENT) == 0) {
				*pde = calloc_ram(__PAGESIZE, __PAGESIZE) | flags | X86_PTE_WRITE | X86_PTE_USER;
			}
			pte = (pte64_t *)((uintptr_t)*pde & ~0x3ff);
			pte[(vaddr >> 12) & 0x1ff] = paddr;
		} else {
			pte32_t			*pte;
			pte32_t			*pde;

			pde = (pte32_t *)(page_tables + ((vaddr >> 22) << 2));
			if((*pde & X86_PTE_PRESENT) == 0) {
				*pde = calloc_ram(__PAGESIZE, __PAGESIZE) | flags | X86_PTE_WRITE | X86_PTE_USER;
			}
			pte = (pte32_t *)(*pde & ~0x3ff);
			pte[(vaddr >> 12) & 0x3ff] = paddr;
		}
		vaddr += __PAGESIZE;
		paddr += __PAGESIZE;
	}
	return(mapped_vaddr);
}

//
// Create a 4M mapping using large pages
//
uintptr_t
x86_4m_map(uintptr_t vaddr, paddr_t paddr, paddr_t size, int flags) {
	uintptr_t		mapped_vaddr = vaddr;

//kprintf("bigmap(%x,%x,%x,%x)\n", vaddr, paddr, size, flags);
	paddr |= flags;

	if(paddr_bits > 32) {
		pte64_t			*pde;

		// Big pages are 2M in size when PAE is used.
		while(size) {
			pde = (pte64_t *)(page_tables + ((vaddr >> 21) << 3));
			*pde = paddr;
			size -= X86_4M_PAGESIZE/2;
			paddr += X86_4M_PAGESIZE/2;
			vaddr += X86_4M_PAGESIZE/2;
		}
	} else {
		pte32_t			*pde;

		while(size) {
			pde = (pte32_t *)(page_tables + ((vaddr >> 22) << 2));
			*pde = paddr;
			size -= X86_4M_PAGESIZE;
			paddr += X86_4M_PAGESIZE;
			vaddr += X86_4M_PAGESIZE;
		}
	}
	return mapped_vaddr;
}

	
//
// Allocate page directory(s) and map the last entry to itself.
// procnto needs this last entry mapped this way. If PAE is enabled,
// allocate 16KB of page directory entries (per CPU) and point the last 4
// entries at the 4 pages of the directory entries.
//
void
x86_init_pgdir() {
	unsigned		num;
	paddr32_t		paddr;
	unsigned		i;
	unsigned		pd_size;

	num = lsp.syspage.p->num_cpu;
	set_syspage_section(&lsp.cpu.pgdir, sizeof(paddr32_t) * num);
	pd_size = (paddr_bits > 32) ? (__PAGESIZE * 4) : __PAGESIZE;
	page_tables = paddr = calloc_ram(pd_size * num, __PAGESIZE);
	for(i = 0; i < num; ++i) {
		if(paddr_bits > 32) {
			unsigned	i;

			//
			// Make last eight entries point at the four page directory
			// tables. Entry -8, -7, -6, -5 cover all of them and
			// entries -4, -3, -2, -1 duplicate the values. The reason
			// for this is so that we can use -8 => -5 as the
			// Page-Directory-Pointer table and -4 => -1 as both the
			// page directory and page table entries for this structure.
			// Using some funky addresses in procnto lets us find
			// entries in the page directory(s) and page tables.
			// We can't use the same entries for both the PDPT and
			// the page directory/page table since the hardware doesn't
			// like it if the access bit is on an a PDPT entry.
			// We lose the top 16M of virtual address space this way
			// rather than 8M if we could use one set of entries for
			// everything, but we're not tight on vaddr's yet.
			//
			for(i = 0; i < 4; ++i) {
				*(pte64_t *)(paddr + (__PAGESIZE*4 - 0x40) + i * 8) =
							(paddr | X86_PTE_PRESENT) + __PAGESIZE * i;
				*(pte64_t *)(paddr + (__PAGESIZE*4 - 0x20) + i * 8) =
							(paddr | X86_PTE_PRESENT) + __PAGESIZE * i;
			}
		} else {
			*(pte32_t *)(paddr + __PAGESIZE-4) = paddr | X86_PTE_PRESENT;
		}
		lsp.cpu.pgdir.p[i] = x86_map(~0L, paddr, pd_size, X86_PTE_PRESENT | X86_PTE_WRITE);
		paddr += pd_size;
	}
}

void
x86_load_pgdir(unsigned cpu) {
	unsigned		pd_size;
	unsigned		cp_size;
	paddr32_t		pg_tbl;

	if(paddr_bits > 32) {
		pd_size = __PAGESIZE * 4;
		cp_size = __PAGESIZE * 4 - 0x40;
	} else {
		pd_size = __PAGESIZE;
		cp_size = __PAGESIZE - 4;
	}

	pg_tbl = page_tables + cpu * pd_size;
	if(cpu != 0) {
		// Copy CPU 0's page dir. All but last entry(s) which are itself.
		memmove((void *)pg_tbl, (void *)page_tables, cp_size);
	}
	if(paddr_bits > 32) pg_tbl += cp_size;
	ldpgdir(pg_tbl);
}


paddr_t
elf_vaddr_to_paddr(uintptr_t vaddr) {
	paddr_t		paddr;
	uintptr_t	mask;

	mask = KILO(4) - 1;
	if(paddr_bits > 32) {
		pte64_t			*pte;
		pte64_t			pde;

		pde = *(pte64_t *)(page_tables + ((vaddr >> 21) << 3));
		if(pde & X86_PDE_PS) {
			paddr = pde;
			mask = MEG(2) - 1;
		} else {
			pte = (pte64_t *)((uintptr_t)pde & ~0x3ff);
			paddr = pte[(vaddr >> 12) & 0x1ff];
		}
	} else {
		pte32_t			*pte;
		pte32_t			pde;

		pde = *(pte32_t *)(page_tables + ((vaddr >> 22) << 2));
		if(pde & X86_PDE_PS) {
			paddr = pde;
			mask = MEG(4) - 1;
		} else {
			pte = (pte32_t *)((uintptr_t)pde & ~0x3ff);
			paddr = pte[(vaddr >> 12) & 0x3ff];
		}
	}
	paddr = (paddr & ~(paddr_t)0x3ff) + (vaddr & mask);
	return paddr;
};

__SRCVERSION("processor32.c $Rev: 1 $");
