/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"

//
// Initialize cycle_per_sec field in the qtime structure in the system page.
//
void
set_cycles_per_sec(struct qtime_entry *qtime) {
	unsigned long 		start_cpu, end_cpu;
	unsigned			ticks;
	unsigned			start_tick;
    uint64_t			rdtsc_ticks;
	unsigned long		rate;
	int					scale;

	if(cycles_freq != 0) {
		qtime->cycles_per_sec = cycles_freq;
		return;
	}

	//
	// If the processor supports the rdtsc opcode we calculate
	// its frequency. Otherwise we use the period of the PC
	// timer chip. Note that the kernel will emulate rdtsc
	// opcodes on processors which do not support it and
	// approximate its value using the timer chip.
	//
	x86_cputype();	// make sure cpu_features is set
	if(cpu_features & X86_FEATURE_TSC) {
		ticks = timer_ns2tick(10000000); /* ticks in 1/100th sec */
		start_tick = timer_start();
		rdtsc(&rdtsc_ticks);
		start_cpu = rdtsc_ticks;
		while(timer_diff(start_tick) < ticks)
			;
		rdtsc(&rdtsc_ticks);
		end_cpu = rdtsc_ticks;
		qtime->cycles_per_sec = (end_cpu - start_cpu) * 100;
	} else {
		rate = qtime->timer_rate;
		scale = (9 + qtime->timer_scale);
		while(scale > 0) {
			rate *= 10;
			--scale;
		}
		while(scale < 0) {
			rate /= 10;
			++scale;
		}
		qtime->cycles_per_sec = 1000000000 / rate;
	}
}

__SRCVERSION("set_cycles.c $Rev: 1 $");
