/*
 * $QNXLicenseC:
 * Copyright 2008, QNX Software Systems. 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */





#include "startup.h"

//
// Create a shadow copy of a PCBIOS ROM in RAM. Should not have to be
// modified by users.
//

int
x86_pcbios_shadow_rom(paddr32_t rom, size_t size) {
	uint8_t		hdr[3];
	unsigned	pgsize;
	unsigned	calc_size;
	paddr32_t	ram;

	memmove(hdr, (void *)rom, sizeof(hdr));
	if(hdr[0] != 0x55 || hdr[1] != 0xaa) {
		return 0;
	}
	if(!(shdr->flags1 & STARTUP_HDR_FLAGS1_VIRTUAL)) {
		return 1;
	}
	calc_size = hdr[2] << 9;
	if(calc_size > size) size = calc_size;
	pgsize = ROUNDPG(size);
	ram = calloc_ram(pgsize, __PAGESIZE);
	if(ram == NULL_PADDR32) {
		crash("No memory for ROM shadow\n");
	}
	x86_map(REAL_VADDR + rom, ram, pgsize, X86_PTE_PRESENT | X86_PTE_WRITE);
	memmove((void *)ram, (void *)rom, size);
	return 2;
}

__SRCVERSION("x86_pcbios_shadow.c $Rev: 1 $");
